@@ -78,8 +78,9 @@
         if (any(noInst))
             .warning(
                 paste(
-                    "package(s) not installed when version(s) same",
-                    "as current; use `force = TRUE` to re-install: \n'%s'"
+                    "package(s) not installed when version(s) same as or",
+                    "greater than current; use `force = TRUE` to re-install: ",
+                    "\n'%s'"
                 ),
                 paste(pkgs[noInst], collapse = "' '")
             )
@@ -142,9 +143,10 @@
 }
 
 .install_github <-
-    function(pkgs, lib, lib.loc, repos, force, ...)
+    function(pkgs, lib, lib.loc, repos, update, ask, force, ...)
 {
     doing <- .install_filter_github_repos(pkgs)
+    ask <- if (!update) "never" else if (update && !ask) "always" else "default"
 
     oopts <- options(repos = repos)     # required by remotes::
     on.exit(options(oopts))
@@ -153,7 +155,9 @@
         .message("Installing github package(s) %s", pkgNames)
         .install_github_load_remotes(pkgs, lib.loc = lib.loc)
         for (repo in doing)
-            remotes::install_github(repo, lib = lib, force = force, ...)
+            remotes::install_github(
+                repo, lib = lib, upgrade = ask, force = force, ...
+            )
     }
     setdiff(pkgs, doing)
 }
@@ -189,17 +193,18 @@
 
 .install <-
     function(pkgs, old_pkgs, instPkgs, repos, lib.loc=NULL, lib=.libPaths()[1],
-        checkBuilt, force, ...)
+        update, ask, force, ...)
 {
     requireNamespace("utils", quietly=TRUE) ||
         .stop("failed to load package 'utils'")
 
     todo <- .install_repos(
         pkgs, old_pkgs, instPkgs = instPkgs, lib = lib, repos = repos,
-        checkBuilt = checkBuilt, force = force, ...
+        force = force, ...
     )
     todo <- .install_github(
-        todo, lib = lib, lib.loc = lib.loc, repos = repos, force = force, ...
+        todo, lib = lib, lib.loc = lib.loc, repos = repos,
+        update = update, ask = ask, force = force, ...
     )
 
     if (length(todo))
@@ -244,7 +249,7 @@
 }
 
 .install_updated_version <-
-    function(valid, update, old_pkgs, instPkgs, repos, ...)
+    function(valid, update, old_pkgs, instPkgs, repos, ask, force, ...)
 {
     if (isTRUE(valid))
         return(valid)
@@ -254,7 +259,10 @@
     if (is.null(pkgs) || !update)
         return(pkgs)
 
-    .install(pkgs, old_pkgs, instPkgs, repos, ...)
+    .install(
+        pkgs, old_pkgs, instPkgs, repos, update = update,
+        ask = ask, force = force, ...
+    )
     pkgs
 }
 
@@ -392,7 +400,7 @@ install <-
 
     cmp <- .version_compare(version, version())
     action <- if (cmp < 0) "Downgrade" else "Upgrade"
-    repos <- .repositories(site_repository, version = version)
+    repos <- .repositories(site_repository, version = version, ...)
 
     vout <- .valid_out_of_date_pkgs(pkgs = inst,
         repos = repos, ..., checkBuilt = checkBuilt,
@@ -423,14 +431,14 @@ install <-
 
     pkgs <- .install(
         pkgs, vout[["out_of_date"]], instPkgs = inst, repos = repos,
-        checkBuilt = checkBuilt, force = force, ...
+        update = update, ask = ask, force = force, ...
     )
     if (update && cmp == 0L) {
         .install_update(repos, ask, checkBuilt = checkBuilt, ...)
     } else if (cmp != 0L) {
         .install_updated_version(
-            valist, update, vout[["out_of_date"]], inst, repos, force = force,
-            ...
+            valist, update, vout[["out_of_date"]], inst, repos, ask = ask,
+            force = force, ...
         )
     }
 
